local revision = tonumber(("$Revision: 63020 $"):match("%d+"))
if not NSD_REVISION or revision > NSD_REVISION then
    NSD_REVISION = revision
    NSD_DATE = ("$Date: 2008-02-27 02:03:53 -0500 (Wed, 27 Feb 2008) $"):match("%d%d%d%d%-%d%d%-%d%d")
end

-- members
AddonBase = {
    addonName = "",
    displayName = "",
    enabled = true,
    activated = false,
    loaded = false,
    backdropNoDrag = "Interface/Tooltips/UI-Tooltip-Background",
    backdropMayDrag = "Interface/Tooltips/ChatBubble-Background"
}

local NShakedown = nil

-- constructor
function AddonBase:new(addonName, displayName, alreadyLoaded)
    -- OO init
    local o = {}
    setmetatable(o, self)
    self.__index = self
    self.__lt = AddonBase.LT
    
    if not NShakedown then
        NShakedown = Rock:GetAddon("NShakedown")
    end
    
    -- set the names
    o.displayName = displayName
    o.addonName = addonName
    
    local valid = false
    
    if alreadyLoaded then
        
        -- addon is ready to go
        o.loaded = true
        valid = true
        
    else
    
        -- is this addon loaded?
        o.loaded = (IsAddOnLoaded(addonName) == 1)
        
        if o.loaded then

            -- add to the list
            valid = true
            
        else
            -- check if this is LOD
            local LOD = IsAddOnLoadOnDemand(addonName)
            
            -- check if this is enabled
            local _, _, _, enabled, loadable, reason = GetAddOnInfo(addonName)
            
            -- see if the addon is enabled and loadable
            if enabled == 1 and (loadable == 1 or reason == "NOT_DEMAND_LOADED" or reason == "DEP_NOT_DEMAND_LOADED") then
            
                if LOD ~= 1 then
                
                    -- addon is not a blizz LOD addon, so just call it loaded as it will load itself later
                    o.loaded = true
                
                end
                
                valid = true
                
            end
            
        end
        
    end
    
    if valid then
        NShakedown:AddAddon(o)
    else
        o = nil
    end
    
    return o
end

-- load an LOD addon, returning true if load was successful
function AddonBase:Load()
    if self.loaded then return true end

    -- attempt to load
    local loaded, _ = LoadAddOn(self.addonName)
    
    -- did we load OK?
    self.loaded = (loaded == 1)
    
    if not self.loaded then
        self.activated = false
    end
    
    if self.loaded then
        if self.LoadActions then
            return self:LoadActions()
        else
            return true
        end
    else
        return false
    end

end

function AddonBase:Activate()
    self.activated = true
    
    -- if it isn't loaded, attempt to load it first
    if (self.loaded or self:Load()) then
        self:Start()
    end
end

function AddonBase:Deactivate()
    self.activated = false
    
    self:Stop()
end

-- compare function, so that we can sort by addon name
function AddonBase:LT(other)
    if self.addonName == "StandardUI" then
        return true
    elseif other.addonName == "StandardUI" then
        return false
    else
        return (self.displayName < other.displayName)
    end
end
