﻿local revision = tonumber(("$Revision: 63470 $"):match("%d+"))
if not NSD_REVISION or revision > NSD_REVISION then
    NSD_REVISION = revision
    NSD_DATE = ("$Date: 2008-03-03 10:24:40 -0500 (Mon, 03 Mar 2008) $"):match("%d%d%d%d%-%d%d%-%d%d")
end

-- localisations
local L = Rock("LibRockLocale-1.0"):GetTranslationNamespace("NShakedown")
L:AddTranslations("enUS", function() return {
    -- options
    ["Start"] = true,
    ["Start shakedown."] = true,
    ["Stop"] = true,
    ["Stop shakedown."] = true,
    ["Addons"] = true,
    ["Enable/disable shakedown of Addons."] = true,
    ["Enable/disable shakedown of %s."] = true,
    ["Enable All"] = true,
    ["Enable shakedown for all Addons."] = true,
    ["Disable All"] = true,
    ["Disable shakedown for all Addons."] = true,
    
    -- keybinging
    ["Start/stop a shakedown."] = true,
    
	-- fubar
    ["FuBarPlugin Config"] = true,
    ["Configure the FuBar Plugin."] = true,
    ["|cffffff00Click|r to %s shakedown."] = true,
    ["|cffffff00Right-Click|r to change settings."] = true,
} end)

L:AddTranslations("zhCN", function() return {
    -- options
    ["Start"] = "开始",
    ["Start shakedown."] = "开始位置调试",
    ["Stop"] = "停止",
    ["Stop shakedown."] = "停止位置调试",
    ["Addons"] = "插件",
    ["Enable/disable shakedown of Addons."] = "启用/禁用插件显示元素调试",
    ["Enable/disable shakedown of %s."] = "启用/禁用%s的显示元素调试.",
    ["Enable All"] = "启用所有",
    ["Enable shakedown for all Addons."] = "启用所有插件的显示调试",
    ["Disable All"] = "禁用所有",
    ["Disable shakedown for all Addons."] = "禁用所有插件的显示调试",
  
    -- keybinging
    ["Start/stop a shakedown."] = "开始/停止一次界面调试",
    
	-- fubar
    ["FuBarPlugin Config"] = "Fubar组件设置",
    ["Configure the FuBar Plugin."] = "设置Fubar组件",
    ["|cffffff00Click|r to %s shakedown."] = "|cffffff00点击|r开始调试",
    ["|cffffff00Right-Click|r to change settings."] = "|cffffff00右键点击|r打开设置",
} end)

-- constants
local ICON = "Interface\\Addons\\NShakedown\\Icons\\shakedown"
local ICON_ROT = "Interface\\Addons\\NShakedown\\Icons\\shakedown_rot"
local TIMER_FREQ = 0.2
local TIMER_ID = "IconFlipShakedown"

-- create the addon
local NShakedown = Rock:NewAddon("NShakedown", "LibRockDB-1.0", "LibRockConfig-1.0", "LibRockTimer-1.0", "LibRockHook-1.0", "LibFuBarPlugin-3.0")
NShakedown.version = "r" .. NSD_REVISION
NShakedown.revision = NSD_REVISION
NShakedown.date = NSD_DATE

-- set the database
NShakedown:SetDatabase("NShakedownDB")
NShakedown:SetDatabaseDefaults('profile', {
    state = {},
})

-- console
NShakedown:SetConfigSlashCommand("/shakedown", "/nshakedown","/nsd")

-- register the keybindings
BINDING_HEADER_NSHAKEDOWN = "NShakedown"
BINDING_NAME_NSHAKEDOWN_START = L["Start/stop a shakedown."]

-- members
NShakedown.activated = false
NShakedown.enabled = false
NShakedown.rotatedIcon = false
NShakedown.addons = {}

function NShakedown:OnInitialize()

    if self.db.profile.PitBull == nil then
		self.db.profile.PitBull = "raid"
	end
	
	self:SetupOptions()
    
    -- sort the addons alphabetically
    table.sort(self.addons)
    
    local i = 101
    local opt = self.OnMenuRequest.args
    for k, v in ipairs(self.addons) do
        displayName = v.displayName
        
        -- default to activated
        if NShakedown.db.profile.state[displayName] == nil then
            NShakedown.db.profile.state[displayName] = true
        end
        
        -- activate according to the profile
        v.enabled = NShakedown.db.profile.state[displayName]
        
        -- populate the menu
        if v.addonName ~= "StandardUI" then
            opt.enable.args[k] = {
                type = "toggle",
                name = displayName,
                desc = L["Enable/disable shakedown of %s."]:format(displayName),
                get = function()
                    return v.enabled
                end,
                set = function(value)
                    NShakedown.db.profile.state[v.displayName] = value
                    v.enabled = value
                end,
                order = i,
            }
        end
    end
    
end

function NShakedown:OnEnable(first)

    self.enabled = true

end

function NShakedown:OnDisable()

    self:Deactivate()
    self.enabled = false

end

function NShakedown:Activate()

    if not self.enabled then return end

    if self.activated then return end
    
    local ret, error

    self.activated = true
    
    -- start the icon flipping
    self:AddRepeatingTimer(TIMER_ID, TIMER_FREQ, self.ChangeIcon, self)
    
    --make the addons sing and dance
    for k, v in ipairs(self.addons) do
        if v.enabled then
            ret, error = pcall(v.Activate, v)
            if not ret then
                geterrorhandler()(error)
            end
        end
    end
    
end

function NShakedown:Deactivate()

    --stop display
    for k, v in ipairs(self.addons) do
        if v.activated then
            v:Deactivate()
        end
    end
    
    -- stop the icon flipping
    self:RemoveTimer(TIMER_ID)
    NShakedown:SetFuBarIcon(ICON)
    self.rotatedIcon = false
    
    -- reset the menu
    self.OnMenuRequest.args.trigger.name = L["Start"]
    self.OnMenuRequest.args.trigger.desc = L["Start shakedown."]
   
    self.activated = false
end

function NShakedown:Trigger()

    if self.enabled then
        if self.activated then
            self.OnMenuRequest.args.trigger.name = L["Start"]
            self.OnMenuRequest.args.trigger.desc = L["Start shakedown."]
            NShakedown:Deactivate()
        else
            self.OnMenuRequest.args.trigger.name = L["Stop"]
            self.OnMenuRequest.args.trigger.desc = L["Stop shakedown."]
            NShakedown:Activate()
        end
    end
end

function NShakedown:AddAddon(o)
    table.insert(self.addons, o)
end

-- FuBar options
NShakedown:SetFuBarOption('iconPath', ICON)
NShakedown:SetFuBarOption('hasNoText', true)
NShakedown:SetFuBarOption('hideWithoutStandby', true)
NShakedown:SetFuBarOption('independentProfile', true)
NShakedown:SetFuBarOption('cannotDetachTooltip', true)

-- left click the FuBar plugin / minimap icon
function NShakedown:OnFuBarClick(button)
    NShakedown:Trigger()
end

-- tooltip formatting
function NShakedown:OnUpdateFuBarTooltip()
    local action = ""
    
    if self.activated then
        action = L["Stop"]
    else
        action = L["Start"]
    end
    
	GameTooltip:AddLine("NShakedown")
	GameTooltip:AddLine(" ")
	GameTooltip:AddLine(L["|cffffff00Click|r to %s shakedown."]:format(action), 0.2, 1, 0.2)
	GameTooltip:AddLine(L["|cffffff00Right-Click|r to change settings."], 0.2, 1, 0.2)
end

-- swap the icon
function NShakedown:ChangeIcon()

    if self.rotatedIcon then
        NShakedown:SetFuBarIcon(ICON)
    else
        NShakedown:SetFuBarIcon(ICON_ROT)
    end
    
    self.rotatedIcon = not self.rotatedIcon
end

-- option tree
function NShakedown:SetupOptions()
    local options = {
    	name = self.name,
    	desc = self.notes,
    	icon = ICON,
        type = "group",
        args = {
            trigger = {
                name = L["Start"],
                desc = L["Start shakedown."],
                type = "execute",
                func = function()
                    self:Trigger()
                end,
                disabled = function()
                    return not self.enabled
                end,
                order = 1,
            },
            enable = {
                name = L["Addons"],
                desc = L["Enable/disable shakedown of Addons."],
                type = 'group',
                order = 2,
                args = {
                    spacer1 = {
                        type = 'header',
                        order = 200,
                    },
                    enable = {
                        name = L["Enable All"],
                        desc = L["Enable shakedown for all Addons."],
                        type = "execute",
                        func = function()
                            for k, v in pairs(self.OnMenuRequest.args.enable.args) do
                                if v.type == "toggle" then
                                    v.set(true)
                                end
                            end
                        end,
                        order = 201,
                    },
                    disable = {
                        name = L["Disable All"],
                        desc = L["Disable shakedown for all Addons."],
                        type = "execute",
                        func = function()
                            for k, v in pairs(self.OnMenuRequest.args.enable.args) do
                                if v.type == "toggle" then
                                    v.set(false)
                                end
                            end
                        end,
                        order = 202,
                    }
                }
            }
        },
    }
    self:SetConfigTable(options)
    self.OnMenuRequest = options
end